open import Lib.Equality as Eq using (_==_ ; auto)
open import Lib.Maybe
open import Lib.Boolean
open import Lib.Proposition
open import Lib.List
open import Lib.Word
open import Lib.Word-Lemmas
open import Lib.Word-Tactics

open Monoid-Equational
open Rewriting
open Associative
open InContext

open import Lib.MaybeEq
open import Lib.MaybeEq-Instances
open import CosetNF
open import Gate
open import Step4.Rel
open import Step4.S8
open import Step4.S8D

module Step4.KD where


  p16-step : Step-Function Gate Rel
  p16-step (X0-gen :: X0-gen :: xs) = Just (xs , at-head (axiom ax-X0-X0=ε))
  p16-step (CX10-gen :: CX10-gen :: xs) = Just (xs , at-head (axiom ax-CX10-CX10=ε))
  p16-step (CX20-gen :: CX20-gen :: xs) = Just (xs , at-head (axiom ax-CX20-CX20=ε))
  p16-step (CCX0-gen :: CCX0-gen :: xs) = Just (xs , at-head (axiom ax-CCX0-CCX0=ε))
  p16-step (CX10-gen :: X0-gen :: xs) = Just (X0-gen :: CX10-gen :: xs , at-head (axiom ax-CX10-X0=X0-CX10))
  p16-step (CX20-gen :: X0-gen :: xs) = Just (X0-gen :: CX20-gen :: xs , at-head (axiom ax-CX20-X0=X0-CX20))
  p16-step (CX20-gen :: CX10-gen :: xs) = Just (CX10-gen :: CX20-gen :: xs , at-head (axiom ax-CX20-CX10=CX10-CX20))
  p16-step (CCX0-gen :: X0-gen :: xs) = Just (X0-gen :: CCX0-gen :: xs , at-head (axiom ax-CCX0-X0=X0-CCX0))
  p16-step (CCX0-gen :: CX10-gen :: xs) = Just (CX10-gen :: CCX0-gen :: xs , at-head (axiom ax-CCX0-CX10=CX10-CCX0))
  p16-step (CCX0-gen :: CX20-gen :: xs) = Just (CX20-gen :: CCX0-gen :: xs , at-head (axiom ax-CCX0-CX20=CX20-CCX0))
  p16-step _ = Nothing

  module P16 = Rewriting.Step (step-cong p16-step)
  nfp16 : ListNF Rel
  nfp16 = record { listnf = P16.multistep 1000 ; lemma-listnf = P16.lemma-multistep 1000 }

  isP16 : Gate -> Boolean
  isP16 CCX0-gen = true
  isP16 CX10-gen = true
  isP16 CX20-gen = true
  isP16 X0-gen = true
  isP16 _ = false


  spd-conj : let X = Gate in let Γ = Rel in (h n : X) -> Maybe (∃ λ (n' : List X) -> Γ ⊢ gen h • gen n === word-of-list n' • gen h)
  spd-conj X0-gen S0-gen = Just ( S0-gen :: S0-gen :: S0-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X0-S0=S0-S0-S0-iI-X0) )
  spd-conj X0-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-X0-S1=S1-X0) )
  spd-conj X0-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-X0-S2=S2-X0) )
  spd-conj X0-gen CS01-gen = Just ( S1-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-X0-CS01=S1-CS01-CS01-CS01-X0) )
  spd-conj X0-gen CS02-gen = Just ( S2-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-X0-CS02=S2-CS02-CS02-CS02-X0) )
  spd-conj X0-gen CS12-gen = Just ( CS12-gen :: nil , up-to-assoc auto (axiom ax-X0-CS12=CS12-X0) )
  spd-conj X0-gen CCZ-gen = Just ( CS12-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X0-CCZ=CS12-CS12-CCZ-X0) )
  spd-conj X0-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-X0-iI=iI-X0) )
  spd-conj CX10-gen S0-gen = Just ( S0-gen :: S1-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-S0=S0-S1-CS01-CS01-CX10) )
  spd-conj CX10-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-CX10-S1=S1-CX10) )
  spd-conj CX10-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-CX10-S2=S2-CX10) )
  spd-conj CX10-gen CS01-gen = Just ( S1-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-CS01=S1-CS01-CS01-CS01-CX10) )
  spd-conj CX10-gen CS02-gen = Just ( CS02-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX10-CS02=CS02-CS12-CCZ-CX10) )
  spd-conj CX10-gen CS12-gen = Just ( CS12-gen :: nil , up-to-assoc auto (axiom ax-CX10-CS12=CS12-CX10) )
  spd-conj CX10-gen CCZ-gen = Just ( CS12-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX10-CCZ=CS12-CS12-CCZ-CX10) )
  spd-conj CX10-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-CX10-iI=iI-CX10) )
  spd-conj CX20-gen S0-gen = Just ( S0-gen :: S2-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-S0=S0-S2-CS02-CS02-CX20) )
  spd-conj CX20-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-CX20-S1=S1-CX20) )
  spd-conj CX20-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-CX20-S2=S2-CX20) )
  spd-conj CX20-gen CS01-gen = Just ( CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX20-CS01=CS01-CS12-CCZ-CX20) )
  spd-conj CX20-gen CS02-gen = Just ( S2-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-CS02=S2-CS02-CS02-CS02-CX20) )
  spd-conj CX20-gen CS12-gen = Just ( CS12-gen :: nil , up-to-assoc auto (axiom ax-CX20-CS12=CS12-CX20) )
  spd-conj CX20-gen CCZ-gen = Just ( CS12-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX20-CCZ=CS12-CS12-CCZ-CX20) )
  spd-conj CX20-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-CX20-iI=iI-CX20) )
  spd-conj CCX0-gen S0-gen = Just ( S0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S0=S0-CS12-CCZ-CCX0) )
  spd-conj CCX0-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S1=S1-CCX0) )
  spd-conj CCX0-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S2=S2-CCX0) )
  spd-conj CCX0-gen CS01-gen = Just ( CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CS01=CS01-CS12-CCZ-CCX0) )
  spd-conj CCX0-gen CS02-gen = Just ( CS02-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CS02=CS02-CS12-CCZ-CCX0) )
  spd-conj CCX0-gen CS12-gen = Just ( CS12-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CS12=CS12-CCX0) )
  spd-conj CCX0-gen CCZ-gen = Just ( CS12-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CCZ=CS12-CS12-CCZ-CCX0) )
  spd-conj CCX0-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-CCX0-iI=iI-CCX0) )
  spd-conj _ _ = Nothing

  module SPD = SemiDirect isP16 isD nfp16 nfd group-like spd-conj


  spde1-act : let X = Gate in let Γ = Rel in (g : X) -> (rep : List X) ->  Maybe (∃ λ (rep' : List X) -> ∃ λ (g' : List X) -> Γ ⊢ gen g • word-of-list rep === word-of-list (rep' ++ g'))
  spde1-act CCK'-gen nil = Just ( CCK'-gen :: nil , nil , refl )
  spde1-act CCK'-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , nil , refl )
  spde1-act CCK'-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( nil , CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CCK'-CCK'-CCK'=CS12-CS12) )
  spde1-act X0-gen nil = Just ( nil , X0-gen :: nil , refl )
  spde1-act X0-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , X0-gen :: CCX0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X0-CCK'=CCK'-X0-CCX0-CCZ) )
  spde1-act X0-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , X0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X0-CCK'-CCK'=CCK'-CCK'-X0-CS12-CCZ) )
  spde1-act CX10-gen nil = Just ( nil , CX10-gen :: nil , refl )
  spde1-act CX10-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CX10-gen :: CCX0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX10-CCK'=CCK'-CX10-CCX0-CCZ) )
  spde1-act CX10-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CX10-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX10-CCK'-CCK'=CCK'-CCK'-CX10-CS12-CCZ) )
  spde1-act CX20-gen nil = Just ( nil , CX20-gen :: nil , refl )
  spde1-act CX20-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CX20-gen :: CCX0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX20-CCK'=CCK'-CX20-CCX0-CCZ) )
  spde1-act CX20-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CX20-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX20-CCK'-CCK'=CCK'-CCK'-CX20-CS12-CCZ) )
  spde1-act CCX0-gen nil = Just ( nil , CCX0-gen :: nil , refl )
  spde1-act CCX0-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CCK'=CCK'-CCZ) )
  spde1-act CCX0-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CCX0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CCK'-CCK'=CCK'-CCK'-CCX0-CS12-CCZ) )
  spde1-act S0-gen nil = Just ( nil , S0-gen :: nil , refl )
  spde1-act S0-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , S0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-S0-CCK'=CCK'-CCK'-S0-CS12-CCZ) )
  spde1-act S0-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CCX0-gen :: S0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-S0-CCK'-CCK'=CCK'-CCX0-S0-CCZ) )
  spde1-act S1-gen nil = Just ( nil , S1-gen :: nil , refl )
  spde1-act S1-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-CCK'=CCK'-S1) )
  spde1-act S1-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-CCK'-CCK'=CCK'-CCK'-S1) )
  spde1-act S2-gen nil = Just ( nil , S2-gen :: nil , refl )
  spde1-act S2-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-CCK'=CCK'-S2) )
  spde1-act S2-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-CCK'-CCK'=CCK'-CCK'-S2) )
  spde1-act CS01-gen nil = Just ( nil , CS01-gen :: nil , refl )
  spde1-act CS01-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS01-CCK'=CCK'-CCK'-CS01-CS12-CCZ) )
  spde1-act CS01-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CCX0-gen :: CS01-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS01-CCK'-CCK'=CCK'-CCX0-CS01-CCZ) )
  spde1-act CS02-gen nil = Just ( nil , CS02-gen :: nil , refl )
  spde1-act CS02-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CS02-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS02-CCK'=CCK'-CCK'-CS02-CS12-CCZ) )
  spde1-act CS02-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CCX0-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS02-CCK'-CCK'=CCK'-CCX0-CS02-CCZ) )
  spde1-act CS12-gen nil = Just ( nil , CS12-gen :: nil , refl )
  spde1-act CS12-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-CCK'=CCK'-CS12) )
  spde1-act CS12-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-CCK'-CCK'=CCK'-CCK'-CS12) )
  spde1-act CCZ-gen nil = Just ( nil , CCZ-gen :: nil , refl )
  spde1-act CCZ-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , CCX0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCZ-CCK'=CCK'-CCX0-CS12-CCZ) )
  spde1-act CCZ-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-CCK'-CCK'=CCK'-CCK'-CCX0) )
  spde1-act iI-gen nil = Just ( nil , iI-gen :: nil , refl )
  spde1-act iI-gen (CCK'-gen :: nil ) = Just ( CCK'-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-CCK'=CCK'-iI) )
  spde1-act iI-gen (CCK'-gen :: CCK'-gen :: nil ) = Just ( CCK'-gen :: CCK'-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-CCK'-CCK'=CCK'-CCK'-iI) )
  spde1-act _ _ = Nothing


  module E1 = CosAct (record { listnf = SPD.nfnh' ; lemma-listnf = SPD.lemma-nfnh' }) spde1-act (\ x y -> Nothing)

  spde2-act : let X = Gate in let Γ = Rel in (g : X) -> (rep : List X) ->  Maybe (∃ λ (rep' : List X) -> ∃ λ (g' : List X) -> Γ ⊢ gen g • word-of-list rep === word-of-list (rep' ++ g'))
  spde2-act CK10-gen nil = Just ( CK10-gen :: nil , nil , refl )
  spde2-act CK10-gen (CK10-gen :: nil ) = Just ( nil , S1-gen :: S1-gen :: S1-gen :: nil , up-to-assoc auto (axiom ax-CK10-CK10=S1-S1-S1) )
  spde2-act CK10-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CX10-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CK10-S0-CK10=S0-CK10-CX10-CS01-CS01-CS01) )
  spde2-act CCK'-gen nil = Just ( nil , CCK'-gen :: nil , refl )
  spde2-act CCK'-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CCK'-CK10=CK10-CCK'-CCK'-CCX0-CS12) )
  spde2-act CCK'-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CCK'-gen :: nil , up-to-assoc auto (axiom ax-CCK'-S0-CK10=S0-CK10-CCK') )
  spde2-act X0-gen nil = Just ( nil , X0-gen :: nil , refl )
  spde2-act X0-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , X0-gen :: CX10-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-X0-CK10=CK10-X0-CX10-CS01-CS01) )
  spde2-act X0-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , X0-gen :: S0-gen :: S0-gen :: S1-gen :: S1-gen :: iI-gen :: iI-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X0-S0-CK10=S0-CK10-X0-S0-S0-S1-S1-iI-iI-iI) )
  spde2-act CX10-gen nil = Just ( nil , CX10-gen :: nil , refl )
  spde2-act CX10-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-CK10=CK10-CS01-CS01) )
  spde2-act CX10-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CX10-gen :: S1-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-S0-CK10=S0-CK10-CX10-S1-CS01-CS01) )
  spde2-act CX20-gen nil = Just ( nil , CX20-gen :: nil , refl )
  spde2-act CX20-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CX20-gen :: CCX0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX20-CK10=CK10-CX20-CCX0-CCZ) )
  spde2-act CX20-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CX20-gen :: S2-gen :: S2-gen :: S2-gen :: CS02-gen :: CS02-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX20-S0-CK10=S0-CK10-CX20-S2-S2-S2-CS02-CS02-CS12-CS12) )
  spde2-act CCX0-gen nil = Just ( nil , CCX0-gen :: nil , refl )
  spde2-act CCX0-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CK10=CK10-CCZ) )
  spde2-act CCX0-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CCX0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S0-CK10=S0-CK10-CCX0-CS12-CCZ) )
  spde2-act S0-gen nil = Just ( nil , S0-gen :: nil , refl )
  spde2-act S0-gen (CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , nil , refl )
  spde2-act S0-gen (S0-gen :: CK10-gen :: nil ) = Just ( CK10-gen :: nil , CX10-gen :: S0-gen :: S0-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-S0-S0-CK10=CK10-CX10-S0-S0-CS01-CS01) )
  spde2-act S1-gen nil = Just ( nil , S1-gen :: nil , refl )
  spde2-act S1-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-CK10=CK10-S1) )
  spde2-act S1-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-S0-CK10=S0-CK10-S1) )
  spde2-act S2-gen nil = Just ( nil , S2-gen :: nil , refl )
  spde2-act S2-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-CK10=CK10-S2) )
  spde2-act S2-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-S0-CK10=S0-CK10-S2) )
  spde2-act CS01-gen nil = Just ( nil , CS01-gen :: nil , refl )
  spde2-act CS01-gen (CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , S0-gen :: S0-gen :: S0-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CS01-CK10=S0-CK10-S0-S0-S0-CS01) )
  spde2-act CS01-gen (S0-gen :: CK10-gen :: nil ) = Just ( CK10-gen :: nil , CX10-gen :: S0-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CS01-S0-CK10=CK10-CX10-S0-CS01-CS01-CS01) )
  spde2-act CS02-gen nil = Just ( nil , CS02-gen :: nil , refl )
  spde2-act CS02-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: CS12-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS02-CK10=CK10-CCK'-CCK'-CCX0-CS12-CS12-CS02-CCZ) )
  spde2-act CS02-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: CS12-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS02-S0-CK10=S0-CK10-CCK'-CCK'-CCX0-CS12-CS12-CS02-CCZ) )
  spde2-act CS12-gen nil = Just ( nil , CS12-gen :: nil , refl )
  spde2-act CS12-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-CK10=CK10-CS12) )
  spde2-act CS12-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-S0-CK10=S0-CK10-CS12) )
  spde2-act CCZ-gen nil = Just ( nil , CCZ-gen :: nil , refl )
  spde2-act CCZ-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-CK10=CK10-CCX0) )
  spde2-act CCZ-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-S0-CK10=S0-CK10-CCX0) )
  spde2-act iI-gen nil = Just ( nil , iI-gen :: nil , refl )
  spde2-act iI-gen (CK10-gen :: nil ) = Just ( CK10-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-CK10=CK10-iI) )
  spde2-act iI-gen (S0-gen :: CK10-gen :: nil ) = Just ( S0-gen :: CK10-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-S0-CK10=S0-CK10-iI) )
  spde2-act _ _ = Nothing

  module E2 = CosAct (record { listnf = E1.lactnf ; lemma-listnf = E1.lemma-lactnf }) spde2-act (\ x y -> Nothing)

  spde3-act : let X = Gate in let Γ = Rel in (g : X) -> (rep : List X) ->  Maybe (∃ λ (rep' : List X) -> ∃ λ (g' : List X) -> Γ ⊢ gen g • word-of-list rep === word-of-list (rep' ++ g'))
  spde3-act CK20-gen nil = Just ( CK20-gen :: nil , nil , refl )
  spde3-act CK20-gen (CK20-gen :: nil ) = Just ( nil , S2-gen :: S2-gen :: S2-gen :: nil , up-to-assoc auto (axiom ax-CK20-CK20=S2-S2-S2) )
  spde3-act CK20-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CX20-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CK20-S0-CK20=S0-CK20-CX20-CS02-CS02-CS02) )
  spde3-act CK10-gen nil = Just ( nil , CK10-gen :: nil , refl )
  spde3-act CK10-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CK10-gen :: nil , up-to-assoc auto (axiom ax-CK10-CK20=CK20-CK10) )
  spde3-act CK10-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , S0-gen :: CK10-gen :: CCK'-gen :: CX10-gen :: S0-gen :: S0-gen :: S0-gen :: CS01-gen :: CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CK10-S0-CK20=S0-CK20-S0-CK10-CCK'-CX10-S0-S0-S0-CS01-CS01-CS12-CCZ) )
  spde3-act CCK'-gen nil = Just ( nil , CCK'-gen :: nil , refl )
  spde3-act CCK'-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CCK'-CK20=CK20-CCK'-CCK'-CCX0-CS12) )
  spde3-act CCK'-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CCK'-gen :: nil , up-to-assoc auto (axiom ax-CCK'-S0-CK20=S0-CK20-CCK') )
  spde3-act X0-gen nil = Just ( nil , X0-gen :: nil , refl )
  spde3-act X0-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , X0-gen :: CX20-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-X0-CK20=CK20-X0-CX20-CS02-CS02) )
  spde3-act X0-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , X0-gen :: S0-gen :: S0-gen :: S2-gen :: S2-gen :: iI-gen :: iI-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X0-S0-CK20=S0-CK20-X0-S0-S0-S2-S2-iI-iI-iI) )
  spde3-act CX10-gen nil = Just ( nil , CX10-gen :: nil , refl )
  spde3-act CX10-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CX10-gen :: CCX0-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX10-CK20=CK20-CX10-CCX0-CCZ) )
  spde3-act CX10-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CX10-gen :: S1-gen :: S1-gen :: S1-gen :: CS01-gen :: CS01-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX10-S0-CK20=S0-CK20-CX10-S1-S1-S1-CS01-CS01-CS12-CS12) )
  spde3-act CX20-gen nil = Just ( nil , CX20-gen :: nil , refl )
  spde3-act CX20-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-CK20=CK20-CS02-CS02) )
  spde3-act CX20-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CX20-gen :: S2-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-S0-CK20=S0-CK20-CX20-S2-CS02-CS02) )
  spde3-act CCX0-gen nil = Just ( nil , CCX0-gen :: nil , refl )
  spde3-act CCX0-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-CK20=CK20-CCZ) )
  spde3-act CCX0-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CCX0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S0-CK20=S0-CK20-CCX0-CS12-CCZ) )
  spde3-act S0-gen nil = Just ( nil , S0-gen :: nil , refl )
  spde3-act S0-gen (CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , nil , refl )
  spde3-act S0-gen (S0-gen :: CK20-gen :: nil ) = Just ( CK20-gen :: nil , CX20-gen :: S0-gen :: S0-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-S0-S0-CK20=CK20-CX20-S0-S0-CS02-CS02) )
  spde3-act S1-gen nil = Just ( nil , S1-gen :: nil , refl )
  spde3-act S1-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-CK20=CK20-S1) )
  spde3-act S1-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-S0-CK20=S0-CK20-S1) )
  spde3-act S2-gen nil = Just ( nil , S2-gen :: nil , refl )
  spde3-act S2-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-CK20=CK20-S2) )
  spde3-act S2-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-S0-CK20=S0-CK20-S2) )
  spde3-act CS01-gen nil = Just ( nil , CS01-gen :: nil , refl )
  spde3-act CS01-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: CS12-gen :: CS01-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS01-CK20=CK20-CCK'-CCK'-CCX0-CS12-CS12-CS01-CCZ) )
  spde3-act CS01-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: CS12-gen :: CS01-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CS01-S0-CK20=S0-CK20-CCK'-CCK'-CCX0-CS12-CS12-CS01-CCZ) )
  spde3-act CS02-gen nil = Just ( nil , CS02-gen :: nil , refl )
  spde3-act CS02-gen (CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , S0-gen :: S0-gen :: S0-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CS02-CK20=S0-CK20-S0-S0-S0-CS02) )
  spde3-act CS02-gen (S0-gen :: CK20-gen :: nil ) = Just ( CK20-gen :: nil , CX20-gen :: S0-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CS02-S0-CK20=CK20-CX20-S0-CS02-CS02-CS02) )
  spde3-act CS12-gen nil = Just ( nil , CS12-gen :: nil , refl )
  spde3-act CS12-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-CK20=CK20-CS12) )
  spde3-act CS12-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-S0-CK20=S0-CK20-CS12) )
  spde3-act CCZ-gen nil = Just ( nil , CCZ-gen :: nil , refl )
  spde3-act CCZ-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-CK20=CK20-CCX0) )
  spde3-act CCZ-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-S0-CK20=S0-CK20-CCX0) )
  spde3-act iI-gen nil = Just ( nil , iI-gen :: nil , refl )
  spde3-act iI-gen (CK20-gen :: nil ) = Just ( CK20-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-CK20=CK20-iI) )
  spde3-act iI-gen (S0-gen :: CK20-gen :: nil ) = Just ( S0-gen :: CK20-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-S0-CK20=S0-CK20-iI) )
  spde3-act _ _ = Nothing

  module E3 = CosAct (record { listnf = E2.lactnf ; lemma-listnf = E2.lemma-lactnf }) spde3-act (\ x y -> Nothing)

  spde4-act : let X = Gate in let Γ = Rel in (g : X) -> (rep : List X) ->  Maybe (∃ λ (rep' : List X) -> ∃ λ (g' : List X) -> Γ ⊢ gen g • word-of-list rep === word-of-list (rep' ++ g'))
  spde4-act K0-gen nil = Just ( K0-gen :: nil , nil , refl )
  spde4-act K0-gen (K0-gen :: nil ) = Just ( nil , iI-gen :: iI-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-K0-K0=iI-iI-iI) )
  spde4-act K0-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , X0-gen :: S0-gen :: S0-gen :: S0-gen :: nil , up-to-assoc auto (axiom ax-K0-S0-K0=S0-K0-X0-S0-S0-S0) )
  spde4-act CK20-gen nil = Just ( nil , CK20-gen :: nil , refl )
  spde4-act CK20-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CK20-gen :: nil , up-to-assoc auto (axiom ax-CK20-K0=K0-CK20) )
  spde4-act CK20-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CX20-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CK20-S0-K0=S0-K0-CX20-CS02-CS02-CS02) )
  spde4-act CK10-gen nil = Just ( nil , CK10-gen :: nil , refl )
  spde4-act CK10-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CK10-gen :: nil , up-to-assoc auto (axiom ax-CK10-K0=K0-CK10) )
  spde4-act CK10-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CX10-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CK10-S0-K0=S0-K0-CX10-CS01-CS01-CS01) )
  spde4-act CCK'-gen nil = Just ( nil , CCK'-gen :: nil , refl )
  spde4-act CCK'-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CCK'-gen :: CCK'-gen :: CCX0-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CCK'-K0=K0-CCK'-CCK'-CCX0-CS12) )
  spde4-act CCK'-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CCK'-gen :: nil , up-to-assoc auto (axiom ax-CCK'-S0-K0=S0-K0-CCK') )
  spde4-act X0-gen nil = Just ( nil , X0-gen :: nil , refl )
  spde4-act X0-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , S0-gen :: S0-gen :: nil , up-to-assoc auto (axiom ax-X0-K0=K0-S0-S0) )
  spde4-act X0-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , X0-gen :: S0-gen :: S0-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X0-S0-K0=S0-K0-X0-S0-S0-iI) )
  spde4-act CX10-gen nil = Just ( nil , CX10-gen :: nil , refl )
  spde4-act CX10-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-K0=K0-CS01-CS01) )
  spde4-act CX10-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CX10-gen :: S1-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-CX10-S0-K0=S0-K0-CX10-S1-CS01-CS01) )
  spde4-act CX20-gen nil = Just ( nil , CX20-gen :: nil , refl )
  spde4-act CX20-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-K0=K0-CS02-CS02) )
  spde4-act CX20-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CX20-gen :: S2-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-CX20-S0-K0=S0-K0-CX20-S2-CS02-CS02) )
  spde4-act CCX0-gen nil = Just ( nil , CCX0-gen :: nil , refl )
  spde4-act CCX0-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-K0=K0-CCZ) )
  spde4-act CCX0-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CCX0-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CCX0-S0-K0=S0-K0-CCX0-CS12-CCZ) )
  spde4-act S0-gen nil = Just ( nil , S0-gen :: nil , refl )
  spde4-act S0-gen (K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , nil , refl )
  spde4-act S0-gen (S0-gen :: K0-gen :: nil ) = Just ( K0-gen :: nil , X0-gen :: nil , up-to-assoc auto (axiom ax-S0-S0-K0=K0-X0) )
  spde4-act S1-gen nil = Just ( nil , S1-gen :: nil , refl )
  spde4-act S1-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-K0=K0-S1) )
  spde4-act S1-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , S1-gen :: nil , up-to-assoc auto (axiom ax-S1-S0-K0=S0-K0-S1) )
  spde4-act S2-gen nil = Just ( nil , S2-gen :: nil , refl )
  spde4-act S2-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-K0=K0-S2) )
  spde4-act S2-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , S2-gen :: nil , up-to-assoc auto (axiom ax-S2-S0-K0=S0-K0-S2) )
  spde4-act CS01-gen nil = Just ( nil , CS01-gen :: nil , refl )
  spde4-act CS01-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , S0-gen :: CK10-gen :: CX10-gen :: S0-gen :: S0-gen :: S0-gen :: S1-gen :: nil , up-to-assoc auto (axiom ax-CS01-K0=K0-S0-CK10-CX10-S0-S0-S0-S1) )
  spde4-act CS01-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , S0-gen :: CK10-gen :: CX10-gen :: S0-gen :: S0-gen :: S0-gen :: S1-gen :: nil , up-to-assoc auto (axiom ax-CS01-S0-K0=S0-K0-S0-CK10-CX10-S0-S0-S0-S1) )
  spde4-act CS02-gen nil = Just ( nil , CS02-gen :: nil , refl )
  spde4-act CS02-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , S0-gen :: CK20-gen :: CX20-gen :: S0-gen :: S0-gen :: S0-gen :: S2-gen :: nil , up-to-assoc auto (axiom ax-CS02-K0=K0-S0-CK20-CX20-S0-S0-S0-S2) )
  spde4-act CS02-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , S0-gen :: CK20-gen :: CX20-gen :: S0-gen :: S0-gen :: S0-gen :: S2-gen :: nil , up-to-assoc auto (axiom ax-CS02-S0-K0=S0-K0-S0-CK20-CX20-S0-S0-S0-S2) )
  spde4-act CS12-gen nil = Just ( nil , CS12-gen :: nil , refl )
  spde4-act CS12-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-K0=K0-CS12) )
  spde4-act CS12-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CS12-gen :: nil , up-to-assoc auto (axiom ax-CS12-S0-K0=S0-K0-CS12) )
  spde4-act CCZ-gen nil = Just ( nil , CCZ-gen :: nil , refl )
  spde4-act CCZ-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-K0=K0-CCX0) )
  spde4-act CCZ-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , CCX0-gen :: nil , up-to-assoc auto (axiom ax-CCZ-S0-K0=S0-K0-CCX0) )
  spde4-act iI-gen nil = Just ( nil , iI-gen :: nil , refl )
  spde4-act iI-gen (K0-gen :: nil ) = Just ( K0-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-K0=K0-iI) )
  spde4-act iI-gen (S0-gen :: K0-gen :: nil ) = Just ( S0-gen :: K0-gen :: nil , iI-gen :: nil , up-to-assoc auto (axiom ax-iI-S0-K0=S0-K0-iI) )
  spde4-act _ _ = Nothing

  module E4 = CosAct (record { listnf = E3.lactnf ; lemma-listnf = E3.lemma-lactnf }) spde4-act (\ x y -> Nothing)
  

  p24-spde4-conj : let X = Gate in let Γ = Rel in (h n : X) -> Maybe (∃ λ (n' : List X) -> Γ ⊢ gen h • gen n === word-of-list n' • gen h)
  p24-spde4-conj X1-gen K0-gen = Just ( K0-gen :: nil , up-to-assoc auto (axiom ax-X1-K0=K0-X1) )
  p24-spde4-conj X1-gen CK20-gen = Just ( CK20-gen :: nil , up-to-assoc auto (axiom ax-X1-CK20=CK20-X1) )
  p24-spde4-conj X1-gen CK10-gen = Just ( K0-gen :: CK10-gen :: S1-gen :: nil , up-to-assoc auto (axiom ax-X1-CK10=K0-CK10-S1-X1) )
  p24-spde4-conj X1-gen CCK'-gen = Just ( CK20-gen :: CCK'-gen :: CS02-gen :: CS02-gen :: CS02-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X1-CCK'=CK20-CCK'-CS02-CS02-CS02-CS12-CCZ-X1) )
  p24-spde4-conj X1-gen X0-gen = Just ( X0-gen :: nil , up-to-assoc auto (axiom ax-X1-X0=X0-X1) )
  p24-spde4-conj X1-gen CX10-gen = Just ( X0-gen :: CX10-gen :: nil , up-to-assoc auto (axiom ax-X1-CX10=X0-CX10-X1) )
  p24-spde4-conj X1-gen CX20-gen = Just ( CX20-gen :: nil , up-to-assoc auto (axiom ax-X1-CX20=CX20-X1) )
  p24-spde4-conj X1-gen CCX0-gen = Just ( CX20-gen :: CCX0-gen :: nil , up-to-assoc auto (axiom ax-X1-CCX0=CX20-CCX0-X1) )
  p24-spde4-conj X1-gen S0-gen = Just ( S0-gen :: nil , up-to-assoc auto (axiom ax-X1-S0=S0-X1) )
  p24-spde4-conj X1-gen S1-gen = Just ( S1-gen :: S1-gen :: S1-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X1-S1=S1-S1-S1-iI-X1) )
  p24-spde4-conj X1-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-X1-S2=S2-X1) )
  p24-spde4-conj X1-gen CS01-gen = Just ( S0-gen :: CS01-gen :: CS01-gen :: CS01-gen :: nil , up-to-assoc auto (axiom ax-X1-CS01=S0-CS01-CS01-CS01-X1) )
  p24-spde4-conj X1-gen CS02-gen = Just ( CS02-gen :: nil , up-to-assoc auto (axiom ax-X1-CS02=CS02-X1) )
  p24-spde4-conj X1-gen CS12-gen = Just ( S2-gen :: CS12-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-X1-CS12=S2-CS12-CS12-CS12-X1) )
  p24-spde4-conj X1-gen CCZ-gen = Just ( CS02-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X1-CCZ=CS02-CS02-CCZ-X1) )
  p24-spde4-conj X1-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-X1-iI=iI-X1) )
  p24-spde4-conj X2-gen K0-gen = Just ( K0-gen :: nil , up-to-assoc auto (axiom ax-X2-K0=K0-X2) )
  p24-spde4-conj X2-gen CK20-gen = Just ( K0-gen :: CK20-gen :: S2-gen :: nil , up-to-assoc auto (axiom ax-X2-CK20=K0-CK20-S2-X2) )
  p24-spde4-conj X2-gen CK10-gen = Just ( CK10-gen :: nil , up-to-assoc auto (axiom ax-X2-CK10=CK10-X2) )
  p24-spde4-conj X2-gen CCK'-gen = Just ( CK10-gen :: CCK'-gen :: CS01-gen :: CS01-gen :: CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X2-CCK'=CK10-CCK'-CS01-CS01-CS01-CS12-CCZ-X2) )
  p24-spde4-conj X2-gen X0-gen = Just ( X0-gen :: nil , up-to-assoc auto (axiom ax-X2-X0=X0-X2) )
  p24-spde4-conj X2-gen CX10-gen = Just ( CX10-gen :: nil , up-to-assoc auto (axiom ax-X2-CX10=CX10-X2) )
  p24-spde4-conj X2-gen CX20-gen = Just ( X0-gen :: CX20-gen :: nil , up-to-assoc auto (axiom ax-X2-CX20=X0-CX20-X2) )
  p24-spde4-conj X2-gen CCX0-gen = Just ( CX10-gen :: CCX0-gen :: nil , up-to-assoc auto (axiom ax-X2-CCX0=CX10-CCX0-X2) )
  p24-spde4-conj X2-gen S0-gen = Just ( S0-gen :: nil , up-to-assoc auto (axiom ax-X2-S0=S0-X2) )
  p24-spde4-conj X2-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-X2-S1=S1-X2) )
  p24-spde4-conj X2-gen S2-gen = Just ( S2-gen :: S2-gen :: S2-gen :: iI-gen :: nil , up-to-assoc auto (axiom ax-X2-S2=S2-S2-S2-iI-X2) )
  p24-spde4-conj X2-gen CS01-gen = Just ( CS01-gen :: nil , up-to-assoc auto (axiom ax-X2-CS01=CS01-X2) )
  p24-spde4-conj X2-gen CS02-gen = Just ( S0-gen :: CS02-gen :: CS02-gen :: CS02-gen :: nil , up-to-assoc auto (axiom ax-X2-CS02=S0-CS02-CS02-CS02-X2) )
  p24-spde4-conj X2-gen CS12-gen = Just ( S1-gen :: CS12-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-X2-CS12=S1-CS12-CS12-CS12-X2) )
  p24-spde4-conj X2-gen CCZ-gen = Just ( CS01-gen :: CS01-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-X2-CCZ=CS01-CS01-CCZ-X2) )
  p24-spde4-conj X2-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-X2-iI=iI-X2) )
  p24-spde4-conj CX12-gen K0-gen = Just ( K0-gen :: nil , up-to-assoc auto (axiom ax-CX12-K0=K0-CX12) )
  p24-spde4-conj CX12-gen CK20-gen = Just ( CK20-gen :: CK10-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX12-CK20=CK20-CK10-CS12-CX12) )
  p24-spde4-conj CX12-gen CK10-gen = Just ( CK10-gen :: nil , up-to-assoc auto (axiom ax-CX12-CK10=CK10-CX12) )
  p24-spde4-conj CX12-gen CCK'-gen = Just ( CK10-gen :: CCK'-gen :: CS01-gen :: CS01-gen :: CS01-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX12-CCK'=CK10-CCK'-CS01-CS01-CS01-CS12-CCZ-CX12) )
  p24-spde4-conj CX12-gen X0-gen = Just ( X0-gen :: nil , up-to-assoc auto (axiom ax-CX12-X0=X0-CX12) )
  p24-spde4-conj CX12-gen CX10-gen = Just ( CX10-gen :: nil , up-to-assoc auto (axiom ax-CX12-CX10=CX10-CX12) )
  p24-spde4-conj CX12-gen CX20-gen = Just ( CX10-gen :: CX20-gen :: nil , up-to-assoc auto (axiom ax-CX12-CX20=CX10-CX20-CX12) )
  p24-spde4-conj CX12-gen CCX0-gen = Just ( CX10-gen :: CCX0-gen :: nil , up-to-assoc auto (axiom ax-CX12-CCX0=CX10-CCX0-CX12) )
  p24-spde4-conj CX12-gen S0-gen = Just ( S0-gen :: nil , up-to-assoc auto (axiom ax-CX12-S0=S0-CX12) )
  p24-spde4-conj CX12-gen S1-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-CX12-S1=S1-CX12) )
  p24-spde4-conj CX12-gen S2-gen = Just ( S1-gen :: S2-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX12-S2=S1-S2-CS12-CS12-CX12) )
  p24-spde4-conj CX12-gen CS01-gen = Just ( CS01-gen :: nil , up-to-assoc auto (axiom ax-CX12-CS01=CS01-CX12) )
  p24-spde4-conj CX12-gen CS02-gen = Just ( CS01-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX12-CS02=CS01-CS02-CCZ-CX12) )
  p24-spde4-conj CX12-gen CS12-gen = Just ( S1-gen :: CS12-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX12-CS12=S1-CS12-CS12-CS12-CX12) )
  p24-spde4-conj CX12-gen CCZ-gen = Just ( CS01-gen :: CS01-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX12-CCZ=CS01-CS01-CCZ-CX12) )
  p24-spde4-conj CX12-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-CX12-iI=iI-CX12) )
  p24-spde4-conj CX21-gen K0-gen = Just ( K0-gen :: nil , up-to-assoc auto (axiom ax-CX21-K0=K0-CX21) )
  p24-spde4-conj CX21-gen CK20-gen = Just ( CK20-gen :: nil , up-to-assoc auto (axiom ax-CX21-CK20=CK20-CX21) )
  p24-spde4-conj CX21-gen CK10-gen = Just ( CK20-gen :: CK10-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX21-CK10=CK20-CK10-CS12-CX21) )
  p24-spde4-conj CX21-gen CCK'-gen = Just ( CK20-gen :: CCK'-gen :: CS02-gen :: CS02-gen :: CS02-gen :: CS12-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX21-CCK'=CK20-CCK'-CS02-CS02-CS02-CS12-CCZ-CX21) )
  p24-spde4-conj CX21-gen X0-gen = Just ( X0-gen :: nil , up-to-assoc auto (axiom ax-CX21-X0=X0-CX21) )
  p24-spde4-conj CX21-gen CX10-gen = Just ( CX10-gen :: CX20-gen :: nil , up-to-assoc auto (axiom ax-CX21-CX10=CX10-CX20-CX21) )
  p24-spde4-conj CX21-gen CX20-gen = Just ( CX20-gen :: nil , up-to-assoc auto (axiom ax-CX21-CX20=CX20-CX21) )
  p24-spde4-conj CX21-gen CCX0-gen = Just ( CX20-gen :: CCX0-gen :: nil , up-to-assoc auto (axiom ax-CX21-CCX0=CX20-CCX0-CX21) )
  p24-spde4-conj CX21-gen S0-gen = Just ( S0-gen :: nil , up-to-assoc auto (axiom ax-CX21-S0=S0-CX21) )
  p24-spde4-conj CX21-gen S1-gen = Just ( S1-gen :: S2-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX21-S1=S1-S2-CS12-CS12-CX21) )
  p24-spde4-conj CX21-gen S2-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-CX21-S2=S2-CX21) )
  p24-spde4-conj CX21-gen CS01-gen = Just ( CS01-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX21-CS01=CS01-CS02-CCZ-CX21) )
  p24-spde4-conj CX21-gen CS02-gen = Just ( CS02-gen :: nil , up-to-assoc auto (axiom ax-CX21-CS02=CS02-CX21) )
  p24-spde4-conj CX21-gen CS12-gen = Just ( S2-gen :: CS12-gen :: CS12-gen :: CS12-gen :: nil , up-to-assoc auto (axiom ax-CX21-CS12=S2-CS12-CS12-CS12-CX21) )
  p24-spde4-conj CX21-gen CCZ-gen = Just ( CS02-gen :: CS02-gen :: CCZ-gen :: nil , up-to-assoc auto (axiom ax-CX21-CCZ=CS02-CS02-CCZ-CX21) )
  p24-spde4-conj CX21-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-CX21-iI=iI-CX21) )
  p24-spde4-conj Swap12-gen K0-gen = Just ( K0-gen :: nil , up-to-assoc auto (axiom ax-Swap12-K0=K0-Swap12) )
  p24-spde4-conj Swap12-gen CK20-gen = Just ( CK10-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CK20=CK10-Swap12) )
  p24-spde4-conj Swap12-gen CK10-gen = Just ( CK20-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CK10=CK20-Swap12) )
  p24-spde4-conj Swap12-gen CCK'-gen = Just ( CCK'-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CCK'=CCK'-Swap12) )
  p24-spde4-conj Swap12-gen X0-gen = Just ( X0-gen :: nil , up-to-assoc auto (axiom ax-Swap12-X0=X0-Swap12) )
  p24-spde4-conj Swap12-gen CX10-gen = Just ( CX20-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CX10=CX20-Swap12) )
  p24-spde4-conj Swap12-gen CX20-gen = Just ( CX10-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CX20=CX10-Swap12) )
  p24-spde4-conj Swap12-gen CCX0-gen = Just ( CCX0-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CCX0=CCX0-Swap12) )
  p24-spde4-conj Swap12-gen S0-gen = Just ( S0-gen :: nil , up-to-assoc auto (axiom ax-Swap12-S0=S0-Swap12) )
  p24-spde4-conj Swap12-gen S1-gen = Just ( S2-gen :: nil , up-to-assoc auto (axiom ax-Swap12-S1=S2-Swap12) )
  p24-spde4-conj Swap12-gen S2-gen = Just ( S1-gen :: nil , up-to-assoc auto (axiom ax-Swap12-S2=S1-Swap12) )
  p24-spde4-conj Swap12-gen CS01-gen = Just ( CS02-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CS01=CS02-Swap12) )
  p24-spde4-conj Swap12-gen CS02-gen = Just ( CS01-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CS02=CS01-Swap12) )
  p24-spde4-conj Swap12-gen CS12-gen = Just ( CS12-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CS12=CS12-Swap12) )
  p24-spde4-conj Swap12-gen CCZ-gen = Just ( CCZ-gen :: nil , up-to-assoc auto (axiom ax-Swap12-CCZ=CCZ-Swap12) )
  p24-spde4-conj Swap12-gen iI-gen = Just ( iI-gen :: nil , up-to-assoc auto (axiom ax-Swap12-iI=iI-Swap12) )

  p24-spde4-conj _ _ = Nothing


  p24-step : Step-Function Gate Rel
  p24-step (X1-gen :: X1-gen :: xs) = Just (xs , at-head (axiom ax-X1-X1=ε))
  p24-step (X2-gen :: X2-gen :: xs) = Just (xs , at-head (axiom ax-X2-X2=ε))
  p24-step (CX12-gen :: CX12-gen :: xs) = Just (xs , at-head (axiom ax-CX12-CX12=ε))
  p24-step (CX21-gen :: CX21-gen :: xs) = Just (xs , at-head (axiom ax-CX21-CX21=ε))
  p24-step (Swap12-gen :: Swap12-gen :: xs) = Just (xs , at-head (axiom ax-Swap12-Swap12=ε))
  p24-step (Swap12-gen :: X1-gen :: xs) = Just (X2-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-X1=X2-Swap12))
  p24-step (Swap12-gen :: X2-gen :: xs) = Just (X1-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-X2=X1-Swap12))
  p24-step (Swap12-gen :: CX12-gen :: xs) = Just (CX21-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-CX12=CX21-Swap12))
  p24-step (Swap12-gen :: CX21-gen :: xs) = Just (CX12-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-CX21=CX12-Swap12))
  p24-step (CX21-gen :: X1-gen :: xs) = Just (X1-gen :: CX21-gen :: xs , at-head (axiom ax-CX21-X1=X1-CX21))
  p24-step (CX12-gen :: X2-gen :: xs) = Just (X2-gen :: CX12-gen :: xs , at-head (axiom ax-CX12-X2=X2-CX12))
  p24-step (CX12-gen :: X1-gen :: xs) = Just (X1-gen :: X2-gen :: CX12-gen :: xs , at-head (axiom ax-CX12-X1=X1-X2-CX12))
  p24-step (CX21-gen :: X2-gen :: xs) = Just (X1-gen :: X2-gen :: CX21-gen :: xs , at-head (axiom ax-CX21-X2=X1-X2-CX21))
  p24-step (X2-gen :: X1-gen :: xs) = Just (X1-gen :: X2-gen :: xs , at-head (axiom ax-X2-X1=X1-X2))
  p24-step (CX12-gen :: CX21-gen :: CX12-gen :: xs) = Just (Swap12-gen :: xs , at-head (axiom ax-CX12-CX21-CX12=Swap12))
  p24-step (CX21-gen :: CX12-gen :: CX21-gen :: xs) = Just (Swap12-gen :: xs , at-head (axiom ax-CX21-CX12-CX21=Swap12))
  p24-step (CX12-gen :: Swap12-gen :: xs) = Just (CX21-gen :: CX12-gen :: xs , at-head (axiom ax-CX12-Swap12=CX21-CX12))
  p24-step (CX21-gen :: Swap12-gen :: xs) = Just (CX12-gen :: CX21-gen :: xs , at-head (axiom ax-CX21-Swap12=CX12-CX21))
  p24-step _ = Nothing

  module P24 = Rewriting.Step (step-cong p24-step)

  mvD3-step : Step-Function Gate Rel
  mvD3-step (S1-gen :: S0-gen :: xs) = Just (S0-gen :: S1-gen :: xs , at-head (axiom ax-S1-S0=S0-S1))
  mvD3-step (S1-gen :: CK10-gen :: xs) = Just (CK10-gen :: S1-gen :: xs , at-head (axiom ax-S1-CK10=CK10-S1))
  mvD3-step (S1-gen :: CK20-gen :: xs) = Just (CK20-gen :: S1-gen :: xs , at-head (axiom ax-S1-CK20=CK20-S1))
  mvD3-step (S1-gen :: CCK'-gen :: xs) = Just (CCK'-gen :: S1-gen :: xs , at-head (axiom ax-S1-CCK'=CCK'-S1))
  mvD3-step (S1-gen :: K0-gen :: xs) = Just (K0-gen :: S1-gen :: xs , at-head (axiom ax-S1-K0=K0-S1))
  mvD3-step (S2-gen :: S0-gen :: xs) = Just (S0-gen :: S2-gen :: xs , at-head (axiom ax-S2-S0=S0-S2))
  mvD3-step (S2-gen :: CK10-gen :: xs) = Just (CK10-gen :: S2-gen :: xs , at-head (axiom ax-S2-CK10=CK10-S2))
  mvD3-step (S2-gen :: CK20-gen :: xs) = Just (CK20-gen :: S2-gen :: xs , at-head (axiom ax-S2-CK20=CK20-S2))
  mvD3-step (S2-gen :: CCK'-gen :: xs) = Just (CCK'-gen :: S2-gen :: xs , at-head (axiom ax-S2-CCK'=CCK'-S2))
  mvD3-step (S2-gen :: K0-gen :: xs) = Just (K0-gen :: S2-gen :: xs , at-head (axiom ax-S2-K0=K0-S2))
  mvD3-step (CS12-gen :: S0-gen :: xs) = Just (S0-gen :: CS12-gen :: xs , at-head (axiom ax-CS12-S0=S0-CS12))
  mvD3-step (CS12-gen :: CK10-gen :: xs) = Just (CK10-gen :: CS12-gen :: xs , at-head (axiom ax-CS12-CK10=CK10-CS12))
  mvD3-step (CS12-gen :: CK20-gen :: xs) = Just (CK20-gen :: CS12-gen :: xs , at-head (axiom ax-CS12-CK20=CK20-CS12))
  mvD3-step (CS12-gen :: CCK'-gen :: xs) = Just (CCK'-gen :: CS12-gen :: xs , at-head (axiom ax-CS12-CCK'=CCK'-CS12))
  mvD3-step (CS12-gen :: K0-gen :: xs) = Just (K0-gen :: CS12-gen :: xs , at-head (axiom ax-CS12-K0=K0-CS12))
  mvD3-step (Swap12-gen :: S0-gen :: xs) = Just (S0-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-S0=S0-Swap12))
  mvD3-step (Swap12-gen :: CCK'-gen :: xs) = Just (CCK'-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-CCK'=CCK'-Swap12))
  mvD3-step (Swap12-gen :: K0-gen :: xs) = Just (K0-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-K0=K0-Swap12))
  mvD3-step (Swap12-gen :: CK10-gen :: xs) = Just (CK20-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-CK10=CK20-Swap12))
  mvD3-step (Swap12-gen :: CK20-gen :: xs) = Just (CK10-gen :: Swap12-gen :: xs , at-head (axiom ax-Swap12-CK20=CK10-Swap12))
  mvD3-step (CK10-gen :: CK20-gen :: xs) = Just (CK20-gen :: CK10-gen :: xs , at-head (axiom ax-CK10-CK20=CK20-CK10))
  mvD3-step _ = Nothing

  module mvD3 = Rewriting.Step (step-cong mvD3-step)

  isP24 : Gate -> Boolean
  isP24 CX12-gen = true
  isP24 CX21-gen = true
  isP24 X1-gen = true
  isP24 X2-gen = true
  isP24 Swap12-gen = true
  isP24 _ = false

  isk0spd : Gate -> Boolean
  isk0spd CCX0-gen = true
  isk0spd CX10-gen = true
  isk0spd CX20-gen = true
  isk0spd X0-gen = true
  isk0spd CCK'-gen = true
  isk0spd CK10-gen = true
  isk0spd CK20-gen = true
  isk0spd K0-gen = true
  isk0spd x = isD x

  isPD : Gate -> Boolean
  isPD x = isP x or isD x

  nfp24 : ListNF Rel
  nfp24 = record { listnf = P24.multistep 1000 ; lemma-listnf = P24.lemma-multistep 1000 }

  nfk0spd : ListNF Rel
  nfk0spd = record { listnf = E4.lactnf ; lemma-listnf = E4.lemma-lactnf }

  module P24K0D = SemiDirect isP24 isk0spd nfp24 nfk0spd group-like p24-spde4-conj

  nf-p24k0d : ListNF Rel
  nf-p24k0d = record { listnf = P24K0D.nfnh' ; lemma-listnf = P24K0D.lemma-nfnh' }

  nf-kpd : ListNF Rel
  nf-kpd = rep 3 (extend-nf isk0spd nf-p24k0d ∘ extend-nf isPD nf-pd)

  nf-de : ListNF Rel
  nf-de = extend-nf isD nfd


  nf-e4 : ListNF Rel
  nf-e4 = record { listnf = E4.lactnf ; lemma-listnf = E4.lemma-lactnf }

  nf-e3 : ListNF Rel
  nf-e3 = record { listnf = E3.lactnf ; lemma-listnf = E3.lemma-lactnf }

  nf-e2 : ListNF Rel
  nf-e2 = record { listnf = E2.lactnf ; lemma-listnf = E2.lemma-lactnf }

  nf-e1 : ListNF Rel
  nf-e1 = record { listnf = E1.lactnf ; lemma-listnf = E1.lemma-lactnf }

  isSPD : Gate -> Boolean
  isSPD CCX0-gen = true
  isSPD CX10-gen = true
  isSPD CX20-gen = true
  isSPD X0-gen = true
  isSPD x = isD x

  isE1 : Gate -> Boolean
  isE1 CCK'-gen = true
  isE1 x = isSPD x

  isE2 : Gate -> Boolean
  isE2 CK10-gen = true
  isE2 x = isE1 x

  isE3 : Gate -> Boolean
  isE3 CK20-gen = true
  isE3 x = isE2 x

  isE4 : Gate -> Boolean
  isE4 K0-gen = true
  isE4 x = isE3 x

  nf-e4e : ListNF Rel
  nf-e4e = extend-nf isE4 nf-e4

  nf-e3e : ListNF Rel
  nf-e3e = extend-nf isE3 nf-e3


